#!/usr/bin/env python
#-*- coding: iso-8859-15 -*-

#################################################################################
## Coban - Module chauffage                                                    ## 
## Copyright (C) 2008  Stphan GURIN                                          ##
## contact : setthe@tuxfamily.org                                              ##
##                                                                             ##
## This program is free software; you can redistribute it and/or               ##
## modify it under the terms of the GNU General Public License                 ##
## as published by the Free Software Foundation; either version 2              ##
## of the License, or (at your option) any later version.                      ##
##                                                                             ##
## This program is distributed in the hope that it will be useful,             ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of              ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               ##
## GNU General Public License for more details.                                ##
##                                                                             ##
## You should have received a copy of the GNU General Public License           ##
## along with this program; if not, write to the Free Software                 ##
## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. ##
#################################################################################


import wx
import pickle
from modcoban import icone, ValidDigitPoint, testFloat, testFich

############################################################################
##                                                                        ##
##             Module de calculs de puissance calorifique                 ##
##                            Version 0.1.4                               ##
##                                                                        ##
##      - Remise en forme de la fentre (ligne, label...)                 ##
##                                                                        ##
############################################################################

#---------------------------------------------------------------------------

version = "0.1.4"
# fichier de sauvegarde des prfrences
fichierPref = "prefcalo.cob"
# valeurs par dfaut du fichier des sauvegarde des prfrences
valDef = [33,85,99]
# messages d'aide
aidePlancher = (u'Surface de la pice en m2.\n'
                u'* Pour les salles de bains, penser  enlever la\n'
                u'  surface occupe par la baignoire et/ou la douche\n'
                u'* Pour les cuisines, penser  enlever la\n'
                u'  surface occupe par les lments de cuisine')
aideAutre = u'Surface de la pice en m2.'


#---------------------------------------------------------------------------

def testPref(f) :
    test = testFich(f)
    if test == 0 :
        fich = open(f,'w')
        pickle.dump(valDef, fich)
        fich.close()


def lireDef(f):
    """ retourne la liste des valeurs du fichier des prfrences """
    testPref(f)
    fich = open(f, 'r')
    lstPdef = pickle.load(fich)
    fich.close()
    return lstPdef


#---------------------------------------------------------------------------


class Pref(wx.Frame):
    """Fentre des prfrences"""
       
    def __init__(
            self, parent, ID, title, pos=wx.DefaultPosition,
            size=wx.DefaultSize, style=wx.DEFAULT_FRAME_STYLE,
            ):

        wx.Frame.__init__(self, parent, ID, title, pos, size, style)
        self.parent = parent
        
        #----------- Cration du panel ------------------
        panel = wx.Panel(self, -1)
        
        #------- cration du sizer principal -----------------
        gbs = wx.GridBagSizer(hgap=5, vgap=5)
        
        #--------- Lecture des valeurs par dfaut ------------
        lst = lireDef(fichierPref)
        PpannDef = lst[0]
        PplanDef = lst[1]
        Paccudef = lst[2]
        
        #--------- Cration des widgets --------------------
        labelPpann = wx.StaticText(panel, -1, 'Chauffage par panneaux rayonnants :\nPuissance en Watts/m3 :')
        self.Ppann = wx.SpinCtrl(panel, -1, "", size=(50,-1))
        self.Ppann.SetRange(1,150)
        self.Ppann.SetValue(PpannDef)
        
        labelPplan = wx.StaticText(panel, -1, u'Chauffage par cble chauffant :\nPuissance en Watts/m2 :')
        self.Pplan = wx.SpinCtrl(panel, -1, "", size=(50,-1))
        self.Pplan.SetRange(1,150)
        self.Pplan.SetValue(PplanDef)
        
        labelPaccu = wx.StaticText(panel, -1, 'Chauffage par accumulateurs :\nPuissance en Watts/m3 :')
        self.Paccu = wx.SpinCtrl(panel, -1, "", size=(50,-1))
        self.Paccu.SetRange(1,400)
        self.Paccu.SetValue(Paccudef)
        
        btnValid = wx.Button(panel, -1, "Valider")
        btnValid.SetDefault()
        self.Bind(wx.EVT_BUTTON, self.onValid, btnValid)
        
        btnReset = wx.Button(panel, -1, u"Valeurs par dfaut")
        self.Bind(wx.EVT_BUTTON, self.onReset, btnReset)
        
        #-------------- Ajout des wigets au sizer --------------------
        
        gbs.Add(labelPpann, (0,0))
        gbs.Add(labelPplan, (1,0))
        gbs.Add(labelPaccu, (2,0))
        gbs.Add(self.Ppann, (0,1))
        gbs.Add(self.Pplan, (1,1))
        gbs.Add(self.Paccu, (2,1))
        gbs.Add(btnValid, (3,2))
        gbs.Add(btnReset, (3,0))
        
        panel.SetSizerAndFit(gbs)
        self.SetClientSize(panel.GetSize())
    
    def onReset(self,evt):
        self.Ppann.SetValue(valDef[0])
        self.Pplan.SetValue(valDef[1])
        self.Paccu.SetValue(valDef[2])


    def onValid(self, event):
        testPref(fichierPref)
        f= open(fichierPref, "w")
        pan = self.Ppann.GetValue()
        plan = self.Pplan.GetValue()
        accu = self.Paccu.GetValue()
        pickle.dump([pan, plan, accu], f)
        f.close()
        
            
        #--------- Lecture des coefs par dfaut ------------
        lst = lireDef(fichierPref)
        PpannDef = lst[0]
        PplanDef = lst[1]
        Paccudef = lst[2]
        
        # modification dans la fenetre chauffage de l'affichage de la puissance de calcul et du rsultat
        if self.parent.typeChauf.GetStringSelection() == self.parent.listeTypeChauf[0]: # si panneau
            self.parent.labelTypeCalc.SetLabel("%s W/m3"%PpannDef)
        elif self.parent.typeChauf.GetStringSelection() == self.parent.listeTypeChauf[2]: # si accu
            self.parent.labelTypeCalc.SetLabel("%s W/m3"%Paccudef)
        else : # si plancher
            self.parent.labelTypeCalc.SetLabel("%s W/m2"%PplanDef)
        
        self.parent.pCalc.SetLabel('... W')
        
        self.Close(True)
        
        

  
class Chauffage(wx.Frame):
    """Module de calculs de puissance calorifique"""
       
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, title = 'Chauffage version %s'%version, 
                          style=wx.DEFAULT_FRAME_STYLE 
                          ^(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX))
        self.parent = parent
        
        #----------- Cration du panel ------------------
        
        panel =  wx.Panel(self, -1, style = wx.TAB_TRAVERSAL
                     | wx.CLIP_CHILDREN
                     | wx.FULL_REPAINT_ON_RESIZE
                     )
        
        #------- cration du sizer principal -----------------
        
        self.gbs = wx.GridBagSizer(hgap=5, vgap=5)
        
        #------------- Cration de statusbar -----------------
        
        self.CreateStatusBar()
        self.SetStatusText("Calculs de puissances calorifiques")
        
        #-------------- CRATION DES MENUS ---------------------
        
        barreMenu = wx.MenuBar()
        self.menu1 = wx.Menu()
        self.menu1pref = self.menu1.Append(wx.ID_ANY, u"Rglages", 
                                              u"Rglages des paramtres de calcul")
        self.Bind(wx.EVT_MENU, self.onPref, self.menu1pref)
        self.menu1.AppendSeparator()
        self.menu1Quit = self.menu1.Append(wx.ID_ANY, u"Quitter", 
                                              u"Quitter ce module...")
        self.Bind(wx.EVT_MENU, self.onQuitter, self.menu1Quit)
        barreMenu.Append(self.menu1, u"&Prfrences") 
        self.SetMenuBar(barreMenu)
        
        #--------- Lecture des coefs par dfaut ------------
        lst = lireDef(fichierPref)
        PplanDef = lst[1]
        
        #------------- cration des widgets -----------------
        
        # titres
        fontTitre = wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD)
        lblProjet = wx.StaticText(panel, -1, u"Donnes du projet")
        lblProjet.SetFont(fontTitre)
        lblResult = wx.StaticText(panel, -1, u"Rsultats")
        lblResult.SetFont(fontTitre)
        
        # Choix de slection du type de chauffage
        self.listeTypeChauf = ['Panneaux',u'Cble chauffant','Accumulateurs']
        labelType = wx.StaticText(panel, -1, u"Mode de chauffage :")
        self.typeChauf = wx.Choice(panel, -1, (-1, -1), choices = self.listeTypeChauf)
        self.typeChauf.SetStringSelection(self.listeTypeChauf[1])
        self.typeChauf.Bind(wx.EVT_CHOICE, self.onTypeChauf, self.typeChauf)
        
        # Choix de la surface
        labelSurface = wx.StaticText(panel, -1, 'Surface (en m2) :')
        self.surface = wx.TextCtrl(panel, -1, "", validator=ValidDigitPoint())
        self.surface.SetToolTipString(aidePlancher)
        self.bain = wx.CheckBox(panel, -1, "Salle de bain ?")
        self.bain.SetToolTipString(u'Cocher cette case si la pice est une salle de bain')
        
        # Choix hauteur sous plafond
        labelHsp= wx.StaticText(panel, -1, 'H.S.P. (en m) :')
        self.hsp = wx.TextCtrl(panel, -1, "2.5", validator = ValidDigitPoint())
        self.hsp.SetToolTipString(u'Hauteur sous plafond en mtres')
        
        # sparation
        sep = wx.StaticLine(panel, -1)
        
        # Fentre "mode de calcul"
        labelModeCalc = wx.StaticText(panel, -1, u"Mode de calcul : ")
        labelModeCalc.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL,wx.NORMAL))
        self.labelTypeCalc = wx.StaticText(panel, -1, "%s W/m2"%PplanDef)
        self.labelTypeCalc.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL,wx.NORMAL))
        
        # Fentre "Puissance calcule"
        labelPCalc = wx.StaticText(panel, -1, u"Puissance calcule :", style=wx.ALIGN_RIGHT)
        self.pCalc = wx.StaticText(panel, -1, '... W')
        self.pCalc.SetForegroundColour('blue')
        self.pCalc.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))
        
        # bouton Calcul
        self.btnCalc = wx.Button(panel, -1, "calculer")
        self.btnCalc.SetDefault()
        self.btnCalc.SetBackgroundColour('yellow')
        self.Bind(wx.EVT_BUTTON, self.onCalcul, self.btnCalc)
        
        # bouton "quitter"
        self.btnQuit = wx.Button(panel, -1, "Quitter")
        self.Bind(wx.EVT_BUTTON, self.onQuitter, self.btnQuit)
        
        self.Bind(wx.EVT_CLOSE, self.onQuitter)
        
        #-------------- Ajout des wigets au sizer --------------------
        self.gbs.Add(lblProjet, (0,0))
        self.gbs.Add(labelType, (1,0))
        self.gbs.Add(self.typeChauf, (1,1),(1,2),wx.EXPAND)
        self.gbs.Add(labelSurface, (2,0))
        self.gbs.Add(self.surface, (2,1))
        self.gbs.Add(self.bain, (2,2))
        self.gbs.Add(labelHsp, (3,0))
        self.gbs.Add(self.hsp, (3,1))
        self.gbs.Add(self.btnCalc, (4,2))
        self.gbs.Add(sep, (5,0), (1,3),wx.EXPAND)
        self.gbs.Add(lblResult, (6,0))
        self.gbs.Add(labelModeCalc, (7,0))
        self.gbs.Add(self.labelTypeCalc, (7,1))
        self.gbs.Add(labelPCalc, (8,0))
        self.gbs.Add(self.pCalc, (8,1))
        
        self.gbs.Add(self.btnQuit, (9,2))
        
        panel.SetSizerAndFit(self.gbs)
        self.SetClientSize(panel.GetSize())
        
        
        
        
    def onTypeChauf(self, evt):
        """Modifie les paramtres en fonction du type de chauffage"""
        
        #--------- Lecture des coefs par dfaut ------------
        lst = lireDef(fichierPref)
        PpannDef = lst[0]
        PplanDef = lst[1]
        Paccudef = lst[2]
        
        if self.typeChauf.GetStringSelection() != self.listeTypeChauf[1]: # si panneau ou accu
            self.bain.Enable(False)
            self.bain.Set3StateValue(wx.CHK_UNCHECKED)
            self.surface.SetToolTipString(aideAutre)
            if self.typeChauf.GetStringSelection() == self.listeTypeChauf[0]: # si panneau
                self.labelTypeCalc.SetLabel("%s W/m3"%PpannDef)
            else : # si accu
                self.labelTypeCalc.SetLabel("%s W/m3"%Paccudef)
            
        else : # si plancher
            self.bain.Enable(True)
            self.surface.SetToolTipString(aidePlancher)
            self.labelTypeCalc.SetLabel("%s W/m2"%PplanDef)
        
        

    def onPref(self, evt):
        testPref(fichierPref)
        dlg = Pref(self, -1, u"Rglages", size=(350, 200),
                   style=wx.DEFAULT_FRAME_STYLE 
                   ^(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX | wx.MINIMIZE_BOX| wx.CLOSE_BOX))
        icone(dlg)
        dlg.Show(True) 
        
        
        
    def onCalcul (self, evt):
        """Calcul de la puissance calorifique"""
        
        #--------- Rcupration des valeurs saisies -----------
        s = testFloat(self.surface)
        h = testFloat(self.hsp)
        vol = s*h
        chauf = self.typeChauf.GetStringSelection()
        bain = self.bain.Get3StateValue()
        
        #--------- Lecture des coefs par dfaut ------------
        lst = lireDef(fichierPref)
        PpannDef = lst[0]
        PplanDef = lst[1]
        Paccudef = lst[2]
        
        if chauf == self.listeTypeChauf[0]: # panneaux
            p = round(vol*PpannDef)
        if chauf == self.listeTypeChauf[1]: # plancher
            if bain == 0:
                s = s*0.85
                p = int(s*PplanDef)
            else :
                p = int(s*85)
        if chauf == self.listeTypeChauf[2]: # accus
            p = int(vol*Paccudef)
            
        self.pCalc.SetLabel('%s W'%p)
    
    
   
    
    def onQuitter(self, evt):
        """Quitter l'application"""
        self.parent.btnChauff.Enable(True)
        self.parent.menuModuleChauffage.Enable(True)
        self.Destroy()
    